@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2012-2016  Belledonne Communications
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef MS_STUN_H
 #define MS_STUN_H
@@ -47,41 +47,42 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define MS_TURN_METHOD_CHANNEL_BIND      0x009
 
 
-#define MS_STUN_ATTR_MAPPED_ADDRESS      0x0001
-#define MS_STUN_ATTR_RESPONSE_ADDRESS    0x0002 /* Deprecated, now reserved */
-#define MS_STUN_ATTR_CHANGE_REQUEST      0x0003 /* Deprecated, now reserved */
-#define MS_STUN_ATTR_SOURCE_ADDRESS      0x0004 /* Deprecated, now reserved */
-#define MS_STUN_ATTR_CHANGED_ADDRESS     0x0005 /* Deprecated, now reserved */
-#define MS_STUN_ATTR_USERNAME            0x0006
-#define MS_STUN_ATTR_PASSWORD            0x0007 /* Deprecated, now reserved */
-#define MS_STUN_ATTR_MESSAGE_INTEGRITY   0x0008
-#define MS_STUN_ATTR_ERROR_CODE          0x0009
-#define MS_STUN_ATTR_UNKNOWN_ATTRIBUTES  0x000A
-#define MS_STUN_ATTR_REFLECTED_FROM      0x000B
-#define MS_STUN_ATTR_REALM               0x0014
-#define MS_STUN_ATTR_NONCE               0x0015
-#define MS_STUN_ATTR_XOR_MAPPED_ADDRESS  0x0020
+#define MS_STUN_ATTR_MAPPED_ADDRESS           0x0001
+#define MS_STUN_ATTR_RESPONSE_ADDRESS         0x0002 /* Deprecated, now reserved */
+#define MS_STUN_ATTR_CHANGE_REQUEST           0x0003 /* Deprecated, now reserved */
+#define MS_STUN_ATTR_SOURCE_ADDRESS           0x0004 /* Deprecated, now reserved */
+#define MS_STUN_ATTR_CHANGED_ADDRESS          0x0005 /* Deprecated, now reserved */
+#define MS_STUN_ATTR_USERNAME                 0x0006
+#define MS_STUN_ATTR_PASSWORD                 0x0007 /* Deprecated, now reserved */
+#define MS_STUN_ATTR_MESSAGE_INTEGRITY        0x0008
+#define MS_STUN_ATTR_ERROR_CODE               0x0009
+#define MS_STUN_ATTR_UNKNOWN_ATTRIBUTES       0x000A
+#define MS_STUN_ATTR_REFLECTED_FROM           0x000B
+#define MS_STUN_ATTR_REALM                    0x0014
+#define MS_STUN_ATTR_NONCE                    0x0015
+#define MS_STUN_ATTR_XOR_MAPPED_ADDRESS       0x0020
 
-#define MS_STUN_ATTR_SOFTWARE            0x8022
-#define MS_STUN_ATTR_ALTERNATE_SERVER    0x8023
-#define MS_STUN_ATTR_FINGERPRINT         0x8028
+#define MS_STUN_ATTR_SOFTWARE                 0x8022
+#define MS_STUN_ATTR_ALTERNATE_SERVER         0x8023
+#define MS_STUN_ATTR_FINGERPRINT              0x8028
 
-#define MS_TURN_ATTR_CHANNEL_NUMBER      0x000C
-#define MS_TURN_ATTR_LIFETIME            0x000D
-#define MS_TURN_ATTR_BANDWIDTH           0x0010 /* Deprecated, now reserved */
-#define MS_TURN_ATTR_XOR_PEER_ADDRESS    0x0012
-#define MS_TURN_ATTR_DATA                0x0013
-#define MS_TURN_ATTR_XOR_RELAYED_ADDRESS 0x0016
-#define MS_TURN_ATTR_EVEN_PORT           0x0018
-#define MS_TURN_ATTR_REQUESTED_TRANSPORT 0x0019
-#define MS_TURN_ATTR_DONT_FRAGMENT       0x001A
-#define MS_TURN_ATTR_TIMER_VAL           0x0021 /* Deprecated, now reserved */
-#define MS_TURN_ATTR_RESERVATION_TOKEN   0x0022
+#define MS_TURN_ATTR_CHANNEL_NUMBER           0x000C
+#define MS_TURN_ATTR_LIFETIME                 0x000D
+#define MS_TURN_ATTR_BANDWIDTH                0x0010 /* Deprecated, now reserved */
+#define MS_TURN_ATTR_XOR_PEER_ADDRESS         0x0012
+#define MS_TURN_ATTR_DATA                     0x0013
+#define MS_TURN_ATTR_XOR_RELAYED_ADDRESS      0x0016
+#define MS_TURN_ATTR_REQUESTED_ADDRESS_FAMILY 0x0017
+#define MS_TURN_ATTR_EVEN_PORT                0x0018
+#define MS_TURN_ATTR_REQUESTED_TRANSPORT      0x0019
+#define MS_TURN_ATTR_DONT_FRAGMENT            0x001A
+#define MS_TURN_ATTR_TIMER_VAL                0x0021 /* Deprecated, now reserved */
+#define MS_TURN_ATTR_RESERVATION_TOKEN        0x0022
 
-#define MS_ICE_ATTR_PRIORITY             0x0024
-#define MS_ICE_ATTR_USE_CANDIDATE        0x0025
-#define MS_ICE_ATTR_ICE_CONTROLLED       0x8029
-#define MS_ICE_ATTR_ICE_CONTROLLING      0x802A
+#define MS_ICE_ATTR_PRIORITY                  0x0024
+#define MS_ICE_ATTR_USE_CANDIDATE             0x0025
+#define MS_ICE_ATTR_ICE_CONTROLLED            0x8029
+#define MS_ICE_ATTR_ICE_CONTROLLING           0x802A
 
 
 #define MS_STUN_ERROR_CODE_TRY_ALTERNATE                  300
@@ -151,6 +152,7 @@ typedef struct {
 	uint16_t channel_number;
 	uint16_t data_length;
 	uint8_t requested_transport;
+	uint8_t requested_address_family;
 	bool_t include_username_attribute;
 	bool_t has_error_code;
 	bool_t has_message_integrity;
@@ -167,6 +169,7 @@ typedef struct {
 	bool_t has_lifetime;
 	bool_t has_channel_number;
 	bool_t has_requested_transport;
+	bool_t has_requested_address_family;
 } MSStunMessage;
 
 typedef enum {
@@ -184,6 +187,12 @@ typedef enum {
 	MS_TURN_CONTEXT_TYPE_RTCP
 } MSTurnContextType;
 
+typedef enum {
+	MS_TURN_CONTEXT_TRANSPORT_UDP,
+	MS_TURN_CONTEXT_TRANSPORT_TCP,
+	MS_TURN_CONTEXT_TRANSPORT_TLS
+} MSTurnContextTransport;
+
 typedef struct {
 	uint32_t nb_send_indication;
 	uint32_t nb_data_indication;
@@ -204,15 +213,19 @@ typedef struct {
 	char *username;
 	char *password;
 	char *ha1;
+	char *root_certificate;
 	uint32_t lifetime;
 	uint16_t channel_number;
 	MSTurnContextState state;
 	MSTurnContextType type;
+	MSTurnContextTransport transport;
 	MSStunAddress relay_addr;
-	struct sockaddr *turn_server_addr;
+	struct sockaddr_storage turn_server_addr;
 	socklen_t turn_server_addrlen;
 	bool_t force_rtp_sending_via_relay;
 	MSTurnContextStatistics stats;
+	struct _MSTurnTCPClient *turn_tcp_client;
+	char *cn;
 } MSTurnContext;
 
 
@@ -306,6 +319,9 @@ MS2_PUBLIC MSStunMessage * ms_turn_send_indication_create(MSStunAddress peer_add
 MS2_PUBLIC MSStunMessage * ms_turn_channel_bind_request_create(MSStunAddress peer_address, uint16_t channel_number);
 MS2_PUBLIC bool_t ms_stun_message_has_requested_transport(const MSStunMessage *msg);
 MS2_PUBLIC uint8_t ms_stun_message_get_requested_transport(const MSStunMessage *msg);
+MS2_PUBLIC bool_t ms_stun_message_has_requested_address_family(const MSStunMessage *msg);
+MS2_PUBLIC uint8_t ms_stun_message_get_requested_address_family(const MSStunMessage *msg);
+MS2_PUBLIC void ms_stun_message_set_requested_address_family(MSStunMessage *msg, uint8_t family);
 MS2_PUBLIC bool_t ms_stun_message_has_lifetime(const MSStunMessage *msg);
 MS2_PUBLIC uint32_t ms_stun_message_get_lifetime(const MSStunMessage *msg);
 MS2_PUBLIC void ms_stun_message_set_lifetime(MSStunMessage *msg, uint32_t lifetime);
@@ -321,6 +337,9 @@ MS2_PUBLIC void ms_turn_context_destroy(MSTurnContext *context);
 MS2_PUBLIC void ms_turn_context_set_server_addr(MSTurnContext *context, struct sockaddr *addr, socklen_t addrlen);
 MS2_PUBLIC MSTurnContextState ms_turn_context_get_state(const MSTurnContext *context);
 MS2_PUBLIC void ms_turn_context_set_state(MSTurnContext *context, MSTurnContextState state);
+MS2_PUBLIC MSTurnContextTransport ms_turn_get_transport_from_string(const char *transport);
+MS2_PUBLIC MSTurnContextTransport ms_turn_context_get_transport(const MSTurnContext *context);
+MS2_PUBLIC void ms_turn_context_set_transport(MSTurnContext *context, MSTurnContextTransport transport);
 MS2_PUBLIC const char * ms_turn_context_get_realm(const MSTurnContext *context);
 MS2_PUBLIC void ms_turn_context_set_realm(MSTurnContext *context, const char *realm);
 MS2_PUBLIC const char * ms_turn_context_get_nonce(const MSTurnContext *context);
@@ -337,10 +356,22 @@ MS2_PUBLIC uint16_t ms_turn_context_get_channel_number(const MSTurnContext *cont
 MS2_PUBLIC void ms_turn_context_set_channel_number(MSTurnContext *context, uint16_t channel_number);
 MS2_PUBLIC void ms_turn_context_set_allocated_relay_addr(MSTurnContext *context, MSStunAddress relay_addr);
 MS2_PUBLIC void ms_turn_context_set_force_rtp_sending_via_relay(MSTurnContext *context, bool_t force);
+MS2_PUBLIC void ms_turn_context_set_root_certificate(MSTurnContext *context, const char *root_certificate);
+MS2_PUBLIC void ms_turn_context_set_cn(MSTurnContext *context, const char *cn);
 MS2_PUBLIC bool_t ms_turn_context_peer_address_allowed(const MSTurnContext *context, const MSStunAddress *peer_address);
 MS2_PUBLIC void ms_turn_context_allow_peer_address(MSTurnContext *context, const MSStunAddress *peer_address);
 MS2_PUBLIC RtpTransport * ms_turn_context_create_endpoint(MSTurnContext *context);
 
+typedef struct _MSTurnTCPClient MSTurnTCPClient;
+
+MS2_PUBLIC MSTurnTCPClient* ms_turn_tcp_client_new(MSTurnContext *context, bool_t use_ssl, const char* root_certificate_path);
+MS2_PUBLIC void ms_turn_tcp_client_destroy(MSTurnTCPClient *turn_tcp_client);
+
+MS2_PUBLIC void ms_turn_tcp_client_connect(MSTurnTCPClient *turn_tcp_client);
+
+MS2_PUBLIC int ms_turn_tcp_client_recvfrom(MSTurnTCPClient *turn_tcp_client, mblk_t *msg, int flags, struct sockaddr *from, socklen_t *fromlen);
+MS2_PUBLIC int ms_turn_tcp_client_sendto(MSTurnTCPClient *turn_tcp_client, mblk_t *msg, int flags, const struct sockaddr *to, socklen_t tolen);
+
 #ifdef __cplusplus
 }
 #endif
